<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/abstract_rows_screen.php';
require_once 'lib/dune_exception.php';
require_once 'lib/rows_factory.php';
require_once 'lib/gcomps_factory.php';

class TvRowsScreen extends AbstractRowsScreen
    implements UserInputHandler
{
    const ID = 'tv';

    public static function get_media_url_str($section_id,
        $base_section_id=null, $text='')
    {
        $arr = array(
            'screen_id' => self::ID,
            'section_id' => $section_id);
        if (isset($base_section_id))
            $arr['base_section_id'] = $base_section_id;
        if ($text != '')
            $arr['text'] = $text;
        return MediaURL::encode($arr);
    }

    ////////////////////////////////////////////////////////////

    public static $INFO_WIDTH = 800;
    public static $INFO_HEIGHT = 600;
    public static $INFO_LEFT = 100;
    public static $INFO_TOP = 100;
    public static $INFO_BOTTOM = 40;

    public static $VOD_WIDTH = 1280;
    public static $VOD_HEIGHT = 720;

    public static $MAX_ICONS_IN_ROW = 7;

    ////////////////////////////////////////////////////////////

    private $tv;
    private $session;

    private $with_clusters;

    public function __construct($tv, $session)
    {
        $this->tv = $tv;
        $this->session = $session;

        $this->with_clusters = defined('PluginRowsPane::headers');

        parent::__construct(self::ID);
    }

    public function get_action_map(MediaURL $media_url, &$plugin_cookies)
    {
        return array(
            GUI_EVENT_KEY_ENTER => $this->custom_action('enter'),
            GUI_EVENT_KEY_D_BLUE => $this->custom_action('fav'),
            GUI_EVENT_KEY_CLEAR => $this->custom_action('fav_remove'),
            GUI_EVENT_KEY_POPUP_MENU => $this->custom_action('popup'),
            GUI_EVENT_PLUGIN_ROWS_INFO_UPDATE =>
                $this->custom_action('plugin_rows_info_update'),
        );
    }

    private function get_channels_by_ids($ids)
    {
        $this->tv->ensure_channels_loaded();
        $channels = array();
        foreach ($ids as $id)
        {
            $ch = $this->tv->get_channel_by_id($id);
            if ($ch == null)
                continue;
            $channels[] = $ch;
        }
        return $channels;
    }

    private function get_favorite_channels(&$plugin_cookies)
    {
        $this->tv->ensure_channels_loaded();
        $fav_ids = $this->tv->get_fav_channel_ids($plugin_cookies);
        return $this->get_channels_by_ids($fav_ids);
    }

    private function get_group_channels($gid)
    {
        $this->tv->ensure_channels_loaded();
        $g = $this->tv->get_group_by_id($gid);
        if ($g == null)
            return array();
        return $this->get_channels_by_ids($g['channels']);
    }

    private static function split_text_to_words($text)
    {
        $special_chars = array("+", "-", "(", ")", "|");
        $text = str_replace($special_chars, " ", $text);
        $raw_words = explode(" ", $text);
        $words = array();
        foreach ($raw_words as $word)
        {
            $word = trim($word);
            if ($word != '')
                $words[] = $word;
        }
        return $words;
    }

    private static function channel_name_contains_text($name, $text)
    {
        $name = mb_strtolower($name, 'UTF-8');
        $name_words = self::split_text_to_words($name);
        $text_words = self::split_text_to_words($text);
        foreach ($text_words as $word_start)
        {
            $found = false;
            for ($i = 0; $i < count($name_words); $i++)
            {
                $word = $name_words[$i];
                if (strpos($word, $word_start) === 0)
                {
                    $found = true;
                    array_splice($name_words, $i, 1);
                    break;
                }
            }

            if (!$found)
                return false;
        }

        return true;
    }

    private function filter_channels($channels, $text)
    {
        $filtered_channels = array();
        foreach ($channels as $ch) {
            if (self::channel_name_contains_text($ch['name'], $text))
                $filtered_channels[] = $ch;
        }
        return $filtered_channels;
    }

    public function get_rows_pane(MediaURL $media_url, &$plugin_cookies)
    {
        $cache = MainHelper::$cache;
        $cache->ensure_cache_loaded();

        $this->tv->ensure_channels_loaded();

        $section_id = $media_url->section_id;
        $is_fav = $section_id == SectionsHelper::FAV_SECTION_ID;
        $is_search = $section_id == 'search';
        $base_section_id = HD::get($media_url, 'base_section_id');
        $text = HD::get($media_url, 'text', '');

        $section_by_id = $this->tv->get_sections();
        $group_by_id = $this->tv->get_groups();

        if ($is_search)
        {
            $section = HD::get($section_by_id, $base_section_id);
            if (!$section && $base_section_id != 'all')
                $section = HD::get($section_by_id, 'all');
            if (!$section)
            {
                foreach ($section_by_id as $id => $section)
                    break;
            }
        }
        else
        {
            $section = $is_fav ? null : HD::get($section_by_id, $section_id);
        }

        if ($section) {
            $group_ids = $section['categories'];
        } else {
            if (!$is_fav)
                hd_print("tv_rows E: section not found: '$section_id'");
            $group_ids = array();
        }

        if ($is_fav)
            $group_ids = $this->tv->get_fav_category_ids($plugin_cookies);

        $n = count($group_ids);
        hd_print("DEBUG section[$section_id/$text]: creating epf with $n categories");

        $section_lang_code = HD::get($section, 'lang_code', '');

        $groups = array();
        foreach ($group_ids as $gid)
        {
            $g = HD::get($group_by_id, $gid);
            if (!$g) {
                if (!$is_fav)
                    hd_print("tv_rows E: group not found for id: $gid");
                continue;
            }

            if (!HD::get($g, 'show_on_rows_screen', 1))
                continue;

            $preferred_lang_codes = array();
            if (isset($g['preferred_lang_codes']))
                $preferred_lang_codes = $g['preferred_lang_codes'];

            $groups[] = (object) array(
                'id' => $g['id'],
                'caption' => $g['name'],
                'preferred_lang_codes' => $preferred_lang_codes,
                'section_lang_code' => $section_lang_code,
            );
        }

        if (HD::with_list_config())
        {
            $lcfg_section_id = $is_fav ? 'fav' : $section_id;
            $lcfg = ConfigHolder::cats_lcfg_ensure_loaded($lcfg_section_id);
            $this->apply_lcfg_to_groups($groups, $lcfg);
        }

        $TITLE_HPAD = 115;
        $ROW_H = 230;
        $INACTIVE_ROW_H = 190;

        $initial_item_id = null;
        $initial_row_id = null;

        $headers = $this->with_clusters ? array() : null;

        $num_channels = 0;

        $rows = array();

        if (!$this->with_clusters)
            $rows[] = RowsFactory::vgap_row(10, 10);

        $item_ndx = $is_fav || !$this->with_clusters ? -1 : 0;
        for (; $item_ndx < count($groups); $item_ndx++)
        {
            if ($item_ndx == -1)
            {
                $row_name = "__favorites";
                $row_caption = T::key_global('dune_tv_favorites');
                $channels = $this->get_favorite_channels($plugin_cookies);
            }
            else
            {
                $group = $groups[$item_ndx];
                $row_name = strval($group->id);
                $row_caption = $group->caption;
                $channels = $this->get_group_channels($group->id);

                if ($is_search)
                {
                    $c_all = count($channels);
                    $channels = $this->filter_channels($channels, $text);
                    $c_filtered = count($channels);
                    $base_caption = preg_replace('/ \(.*\)$/', '', $row_caption);
                    $row_caption = "$base_caption ($c_filtered/$c_all)";
                }
            }
            
            $header_id = null;
            if ($this->with_clusters && $channels)
            {
                $header_id = $row_name;
                RowsFactory::add_header($headers,
                    $header_id, $row_caption, true);
            }

            $reg_items = array();
            for ($ch_ndx = 0; $ch_ndx < count($channels); $ch_ndx++)
            {
                $ch = $channels[$ch_ndx];
                $image_id = "icon:$row_name:" . $ch['id'];

                $icon_url = HD::enable_caching(
                    $cache->get_channel_icon_url($ch));
                RowsFactory::add_regular_item($reg_items,
                    $image_id, $icon_url, $ch['name'], array());
                $num_channels++;
            }

            if ($reg_items)
            {
                $row_gid = "channels:$row_name";

                // Title row.

                $opts = 0;
                if ($this->with_clusters)
                    $opts = PLUGIN_ROW_OPT_FIRST_IN_CLUSTER;

                if ($is_search && $this->with_clusters)
                {
                    $row = RowsFactory::vgap_row(1, 100, $opts);
                    $row[PluginRow::header_id] = $header_id;
                    $row[PluginRow::id] = "vgap:$item_ndx";
                    $rows[] = $row;
                    $opts = 0;
                }

                $row = RowsFactory::title_row("title:$item_ndx",
                    $row_caption, $row_gid, 1920, 65, $opts, "#FFFFFFE0", 35,
                    $TITLE_HPAD, 12, 3, 1, "#FF606060", "#FF808080");
//                if ($this->with_clusters)
//                    $row[PluginRow::header_id] = $header_id;
                $rows[] = $row;

                // Channel list row.

                $row_h = $ROW_H;
                $row_inactive_h = $INACTIVE_ROW_H;

                for ($i = 0; $i < count($reg_items); $i += self::$MAX_ICONS_IN_ROW)
                {
                    $rndx = intval($i / self::$MAX_ICONS_IN_ROW);
                    $row_id = "channels.$rndx:$row_name";
                    $chunk = array_slice($reg_items, $i, self::$MAX_ICONS_IN_ROW);
                    $rows[] = RowsFactory::regular_row($row_id,
                        $chunk, 'common', null, $header_id, $row_caption,
                        $row_gid, 1920, $row_h, $row_inactive_h, 0,
                        90, 120, 120, true, false,
                        true, 0, 170, 128, "#FF808080");
                }

                // Gap.

                $rows[] = RowsFactory::vgap_row(1, 20);
            }
        }

        hd_print("D section $section_id: $num_channels channels");

        if (!$rows)
        {
            $rows[] = RowsFactory::vgap_row(50);
            $caption = T::t("msg_search_empty");

            $defs = array();
            $defs[] = GCompsFactory::label_v2(
                GCompGeom::place_center(-1, -1, 0, 0), null,
                $caption, 1, "#FFAFAFA0", 60);

            $rows[] = RowsFactory::gcomps_row("single_row",
                $defs, null, 1920, 100);
        }
        else
        {
            $rows[] = RowsFactory::vgap_row(79, 60);
        }

        ///////////////////////////////////////////////////////////////////

        $pane = RowsFactory::pane($rows, $headers,
            RowsFactory::focus(
                GCOMP_FOCUS_DEFAULT_CUT_IMAGE,
                GCOMP_FOCUS_DEFAULT_RECT),
            null,
            true, true, -1, $initial_item_id, $initial_row_id,
            1.0, 0.0, -0.5, 50,
            null
        );

        $min_row_index_for_y2 = 0;

        RowsFactory::pane_set_geometry($pane,
            1920, 1080, 0, 0,
            self::$INFO_HEIGHT, $min_row_index_for_y2,
            self::$INFO_WIDTH,
            self::$INFO_HEIGHT - self::$INFO_TOP - self::$INFO_BOTTOM,
            self::$INFO_LEFT, self::$INFO_TOP,
            self::$VOD_WIDTH, self::$VOD_HEIGHT);

        RowsFactory::pane_set_header_params($pane,
            33, 20, 410, 410);

        $def_params = RowsFactory::variable_params(
            246, $ROW_H, 0,
            210, 210, 10,
            10, '#FFAFAFA0', 28,
            75, 75);
        $sel_params = RowsFactory::variable_params(
            189, $ROW_H, 0,
            231, 231, 0,
            10, '#FFFFFFE0', 28,
            83, 83);
        $inactive_params = RowsFactory::variable_params(
            189, $INACTIVE_ROW_H, 0,
            172, 172, 0,
            0, '#00000000', 28,
            45, 45);

        $params = RowsFactory::item_params(
            $def_params, $sel_params, $inactive_params,
            'plugin_file://icons/loading_channel.png',
            'plugin_file://icons/channel_unset.png',
            3, 3, null, RowsFactory::margins(4, 2, 3, 4));
        
        RowsFactory::set_item_params_template($pane, 'common', $params);

        return $pane;
    }

    private static $lcfg_order_by_id = null;

    public static function cmp_groups_by_lcfg_order($o1, $o2)
    {
        $v1 = self::$lcfg_order_by_id[$o1->id];
        $v2 = self::$lcfg_order_by_id[$o2->id];
        return $v1 - $v2;
    }

    private function apply_lcfg_to_groups(&$groups, $lcfg)
    {
        if (!$groups)
            return;

        hd_print("lcfg D: applying changes: ".count($lcfg->changes));
        foreach ($groups as $key => $g)
        {
            $op = HD::arrget($lcfg->changes, $g->id, '');
            if ($op == '-')
                unset($groups[$key]);
        }

        hd_print("lcfg D: applying order: ".count($lcfg->order));
        if ($lcfg->order)
        {
            foreach ($lcfg->order as $k => $v)
                self::$lcfg_order_by_id[$v] = $k;

            $n = count($lcfg->order);
            foreach ($groups as $k => $g)
            {
                if (!isset(self::$lcfg_order_by_id[$g->id]))
                    self::$lcfg_order_by_id[$g->id] = $n + $k;
            }

            usort($groups, 'TvRowsScreen::cmp_groups_by_lcfg_order');

            self::$lcfg_order_by_id = null;
        }

        $groups = array_values($groups);
    }

    private function make_playback_media_url(&$user_input)
    {
        $section_id = -1;
        $parent_media_url = HD::get($user_input, 'parent_media_url');
        if ($parent_media_url && 0 === strpos($parent_media_url, "section_"))
            $section_id = substr($parent_media_url, 8);

        //hd_print("make_playback_media_url, user_input: " . print_r($user_input, true));
        $control_id = HD::get($user_input, 'control_id', '');
        $item_id_field = 'selected_item_id';
        if ($control_id == 'plugin_rows_info_update')
            $item_id_field = 'item_id';

        $item_id = HD::get($user_input, $item_id_field);
        if (!$item_id)
            return null;

        $arr = explode(':', $item_id);
        if (count($arr) != 3 || $arr[0] != 'icon')
            return null;

        $group_id = $arr[1];
        $ch_id = $arr[2];

        return MediaURL::encode(array(
            'section_id' => $section_id,
            'group_id' => $group_id,
            'channel_id' => $ch_id,
        ));
    }

    private function make_program_info($epg_info)
    {
        $prog = (object) array();
        if (!$epg_info)
            return $prog;

        $start_time = date("H:i", $epg_info->start_tm + MainHelper::$timezone_diff_secs);
        $end_time = date("H:i", $epg_info->end_tm + MainHelper::$timezone_diff_secs);
        $prog->time = "$start_time - $end_time";

        $prog->title = $epg_info->title_and_subtitle;
        $prog->desc = $epg_info->desc;
        $prog->year = $epg_info->year;

        $rating_ru = '';
        if (isset($epg_info->rating_ru))
            $rating_ru = strval($epg_info->rating_ru);

        if ($rating_ru != '' && is_numeric($rating_ru))
            $rating_ru .= '+';

        if ($rating_ru != '')
            $prog->age = $rating_ru;

        if (isset($epg_info->main_category))
            $prog->genre = $epg_info->main_category;

        if (isset($epg_info->country))
            $prog->country = $epg_info->country;

        if (isset($epg_info->main_icon))
            $prog->fanart_url = $epg_info->main_icon;

        return $prog;
    }

    private function do_get_info_children($ch_id, &$plugin_cookies)
    {
        $this->tv->ensure_channels_loaded();
        $ch = $this->tv->get_channel_by_id($ch_id);
        $title = $ch ? $ch['name'] : null;
        if (!$title)
        {
            return array(
                "defs" => array(),
                "fanart_url" => "",
            );
        }

        $epg_info = $this->tv->get_program_info($ch_id, time(), $plugin_cookies);
        $prog = $this->make_program_info($epg_info);

        $tsize = 30;
        $item_h = 40;

        $defs = array();

        ///////////// Title /////////////////

        $nspaces = substr_count($title, ' ');
        $title_len = mb_strlen($title, "UTF-8") * 2 - $nspaces;

        $fsize = 36;
        if ($title_len > 0) {
            $fsize = min(120, (int) (36 * 72 / $title_len));
        }
        $max_lines = $fsize < 36 ? 2 : 1;
        $title_num = $max_lines == 1 ?
            (int) ceil($fsize / 36.0) : 2;
        $fsize = max(36, $fsize);
        $title_h = $item_h * $title_num;

        $y = 0;
        $ldef = GCompsFactory::label(
            GCompGeom::place_top_left(
                self::$INFO_WIDTH,
                $title_h + 2 * 10,
                0, $y - 10),
            null, $title, $max_lines, "#FFFFFFE0", $fsize);
        $ldef[GComponentDef::id] = 'ch_title';
        $defs[] = $ldef;

        $y += $title_h + 12;

        ///////////// start_time, end_time, genre, country, person /////////////////

        $str = '';
        foreach (array('time', 'genre', 'country', 'year', 'age') as $key) {
            $val = HD::get($prog, $key);
            if (!$val)
                continue;
            if ($str)
                $str .= " | ";
            $str .= $val;
        }

        if ($str)
        {
            $defs[] = GCompsFactory::label(
                GCompGeom::place_top_left(
                    self::$INFO_WIDTH, $item_h,
                    0, $y),
                null, "%ext%" . $str, 1, "#FFAFAFA0", $tsize);

            $y += $item_h + 12;
        }

        ///////////// Program ////////////////

        $prog_title = HD::get($prog, 'title');
        if ($prog_title)
        {
            $geom = GCompGeom::place_top_left(
                self::$INFO_WIDTH, $item_h + 6, 0, $y - 3);
            $defs[] = GCompsFactory::label($geom,
                null, $prog_title, 1, "#FFFFFF40", $tsize);
            $y += $item_h;
        }
        else
            $title_num--;

        ///////////// Description ////////////////

        $desc = HD::get($prog, 'desc');
        if ($desc)
        {
            $geom = GCompGeom::place_top_left(
                self::$INFO_WIDTH, -1, 0, $y + 5);
            $defs[] = GCompsFactory::label($geom,
                null, $desc, 8 - $title_num, "#FFAFAFA0", $tsize);
        }

        ///////////// Enclosing panel ////////////////

        $fanart_url = HD::get($prog, 'fanart_url', '');

        $geom = GCompGeom::place_top_left(
            self::$INFO_WIDTH,
            self::$INFO_HEIGHT - self::$INFO_TOP - self::$INFO_BOTTOM,
            0, 0);
        $pane_def = GCompsFactory::get_panel_def("info_pane", $geom,
            null, $defs);
        $props = array('alpha' => 64);
        GCompsFactory::add_extra_var($pane_def,
            "info_inf_dimmed", null, $props);
        $pane_def[GComponentDef::options] = GCOMP_OPT_PREPAINT;
        return array(
            "defs" => array($pane_def),
            "fanart_url" => $fanart_url,
        );
    }

    private function do_handle_user_input(&$user_input, &$plugin_cookies)
    {
        hd_print('TV rows screen: handle_user_input:');
        foreach ($user_input as $key => $value)
            hd_print("  $key => $value");

        $media_url_str = HD::get($user_input, 'media_url');
        if (!$media_url_str)
            $media_url_str = $this->make_playback_media_url($user_input);

        $media_url = $media_url_str ? MediaURL::decode($media_url_str) : null;
        $ch_id = $media_url ? $media_url->channel_id : null;

        $control_id = HD::get($user_input, 'control_id');

        if ($control_id == 'cancel')
            return null;

        if ($control_id == 'plugin_rows_info_update')
        {
            if (!isset($user_input->item_id) || !isset($user_input->folder_key))
                return null;
            $item_id = $user_input->item_id;

            $arr = explode(':', $item_id);
            if (count($arr) != 3 || $arr[0] != 'icon')
                return null;

            $ch_id = $arr[2];
            $favorite = is_numeric($arr[1]) ? 0 : 1;
            $group_id = $favorite ? -2 : $arr[1];

            $this->tv->ensure_channels_loaded();
            $ch = $this->tv->get_channel_by_id($ch_id);
            if ($ch == null)
                return null;

            $is_adult = HD::get($ch, 'adult', false) ||
                0 === strpos($ch['name'], 'XXX');

            $info_children = $this->do_get_info_children($ch_id, $plugin_cookies);

            $fanart_url = HD::get($info_children, "fanart_url");
            if (!$fanart_url)
            {
                $ndx = (HD::get($ch, 'id', 0) + 1) % 4;
                $fanart_url = "plugin_file://%shell_ext%/icons/bg$ndx.jpg";
            }

            $defs = HD::get($info_children, "defs", array());

            $mask_url = "plugin_file://%shell_ext%/icons/mask.png";

            $pf_playback_available = MainHelper::$cache->get_pf_playback_available();
            $video_urls = array();
            if ($pf_playback_available && !$is_adult)
            {
                $media_url->preview = 1;
                $media_url_str = MediaURL::encode($media_url->to_array());
                $video_urls[] = "plugin_tv://globaltv/$media_url_str";
            }

            return ActionFactory::update_rows_info(
                $user_input->folder_key, $user_input->item_id,
                $defs, $fanart_url, null, $mask_url, $video_urls);
        }

        if ($control_id == 'fav' || $control_id == 'fav_remove')
        {
            if ($ch_id == '')
                return null;

            $was_favorite = $this->tv->is_favorite_channel_id(
                $ch_id, $plugin_cookies);
            if (!$was_favorite && $control_id == 'fav_remove')
                return null;
            $fav_op_type = $was_favorite ?
                PLUGIN_FAVORITES_OP_REMOVE : PLUGIN_FAVORITES_OP_ADD;
            $this->tv->change_tv_favorites(
                $fav_op_type, $ch_id, $plugin_cookies);

            $msg = $was_favorite ?
                T::g('tv_channel_removed_from_favorites') :
                T::g('tv_channel_added_to_favorites');
            $post_action = $media_url->group_id == '__favorites' ?
                null : ActionFactory::show_title_dialog(
                    $msg, null,
                    array('params' => array('darkens_screen' => 0)));
            return $this->tv->invalidate_favorites($post_action);
        }

        if ($control_id == 'fav_category')
        {
            $category_id = $user_input->category_id;
            if (!$category_id)
                return null;

            $was_favorite = $this->tv->is_favorite_category_id(
                $category_id, $plugin_cookies);
            $fav_op_type = $was_favorite ?
                PLUGIN_FAVORITES_OP_REMOVE : PLUGIN_FAVORITES_OP_ADD;
            $this->tv->change_tv_favorites_categories(
                $fav_op_type, $category_id, $plugin_cookies);

            $msg = $was_favorite ?
                T::t('category_removed_from_favorites') :
                T::t('category_added_to_favorites');
            $post_action = $user_input->parent_media_url == 'section_fav' ?
                null : ActionFactory::show_title_dialog(
                    $msg, null,
                    array('params' => array('darkens_screen' => 0)));
            return $this->tv->invalidate_favorites($post_action);
        }

        if ($control_id == 'popup')
        {
            if (substr($user_input->parent_media_url, 0, 8) == 'section_')
                $section_id = substr($user_input->parent_media_url, 8);
            else
            {
                $parent_murl = MediaURL::decode($user_input->parent_media_url);
                $section_id = HD::get($parent_murl, 'section_id', '');
            }

            $menu_items = array();

            if ($ch_id != '')
            {
                $was_favorite = $this->tv->is_favorite_channel_id(
                    $ch_id, $plugin_cookies);
                $caption = $was_favorite ?
                    T::g('navigator_popup_remove_from_favorites') :
                    T::g('navigator_popup_add_to_favorites');
                $fav_action = $this->custom_action('fav');
                $menu_items[] = array(
                    GuiMenuItemDef::caption => $caption,
                    GuiMenuItemDef::action => $fav_action);
            }
            else if ('search' != $section_id)
            {
                $selected_row_id = HD::get($user_input, 'selected_row_id');
                $category_id = preg_replace('/^channels\.\d*:/', '', $selected_row_id);

                if ($category_id != "__favorites")
                {
                    $was_favorite = $this->tv->is_favorite_category_id(
                        $category_id, $plugin_cookies);
                    $caption = $was_favorite ?
                        T::g('navigator_popup_remove_from_favorites') :
                        T::g('navigator_popup_add_to_favorites');
                    $fav_action = $this->custom_action(
                        'fav_category', array('category_id' => $category_id));
                    $menu_items[] = ActionFactory::get_menu_item(
                        $caption, $fav_action);
                }
            }

            if ('search' != $section_id)
            {
                $search_section_id = $section_id == 'xxx' ? 'xxx' : 'all';
                $action = MainHelper::$entry_handler->custom_action(
                    'search_channels', array('section_id' => $search_section_id));
                $menu_items[] = ActionFactory::get_menu_item(
                    T::t("popup_action_search_tv_channels"),
                    $action,
                    "plugin_file://%shell_ext%/icons/search_small.png");
            }

            if ('search' != $section_id && HD::with_list_config())
            {
                $hid = HD::get($user_input, 'selected_header_id');
                if (isset($hid))
                {
                    $caption = T::g('navigator_popup_configure_list');
                    $a = MainHelper::$list_helper->edit_cats_list($section_id);
                    $menu_items[] = ActionFactory::get_menu_item($caption, $a, null);
                }
            }

            return ActionFactory::show_popup_menu($menu_items);
        }

        $action = MainHelper::get_pf_check_action_if_needed($user_input, "TV Rows Screen", true);
        if ($action != null)
            return $action;

        $pf_playback_available = true;
        if (isset($user_input->pf_error))
            $pf_playback_available = false;
        MainHelper::$cache->set_pf_playback_available($pf_playback_available);

        if (!$pf_playback_available)
            return null;

        $a = MainHelper::get_pre_playback_action_if_needed($this, $ch_id, $user_input, $plugin_cookies);
        if ($a !== true)
            return $a;

        hd_print("Start media_url: '$media_url_str'");
        return ActionFactory::tv_play($media_url_str);
    }

    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        try
        {
            return $this->do_handle_user_input($user_input, $plugin_cookies);
        }
        catch (Exception $e)
        {
            throw $this->session->dune_api_exception(
                $e, $e->getMessage(), false);
        }
    }

    public function get_folder_view_for_epf($section_id, &$plugin_cookies)
    {
        $media_url_str = TvRowsScreen::get_media_url_str($section_id);
        $media_url = MediaURL::decode($media_url_str);
        return $this->get_folder_view($media_url, $plugin_cookies);
    }
}
?>
